@@ -1,19 +1,37 @@
 import json
+from enum import Enum
 from typing import List
 
 import typer
+import yaml
 
 from globus_automate_client.cli.callbacks import (
-    json_validator_callback,
+    input_validator_callback,
     principal_validator,
     url_validator_callback,
 )
-from globus_automate_client.cli.helpers import format_and_echo, verbosity_option
+from globus_automate_client.cli.constants import InputFormat
+from globus_automate_client.cli.helpers import (
+    format_and_echo,
+    process_input,
+    verbosity_option,
+)
 from globus_automate_client.client_helpers import create_action_client
 
 app = typer.Typer(short_help="Manage Globus Automate Actions")
 
 
+class ActionOutputFormat(str, Enum):
+    json = "json"
+    yaml = "yaml"
+
+    def get_dumper(self):
+        if self is self.json:
+            return json.dumps
+        elif self is self.yaml:
+            return yaml.dump
+
+
 @app.command("introspect")
 def action_introspect(
     action_url: str = typer.Option(
@@ -28,6 +46,14 @@ def action_introspect(
         callback=url_validator_callback,
     ),
     verbose: bool = verbosity_option,
+    output_format: ActionOutputFormat = typer.Option(
+        ActionOutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Introspect an Action Provider's schema.
@@ -35,7 +61,7 @@ def action_introspect(
     ac = create_action_client(action_url, action_scope)
     if ac is not None:
         result = ac.introspect()
-        format_and_echo(result, verbose=verbose)
+        format_and_echo(result, output_format.get_dumper(), verbose=verbose)
     return None
 
 
@@ -61,7 +87,7 @@ def action_run(
             "JSON string."
         ),
         prompt=True,
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     request_id: str = typer.Option(
         None,
@@ -78,15 +104,31 @@ def action_run(
         callback=principal_validator,
     ),
     verbose: bool = verbosity_option,
+    output_format: ActionOutputFormat = typer.Option(
+        ActionOutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
+    input_format: InputFormat = typer.Option(
+        InputFormat.json,
+        "--input",
+        "-i",
+        help="Input format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Launch an Action.
     """
     ac = create_action_client(action_url, action_scope)
     if ac:
-        parsed_body = json.loads(body)
+        parsed_body = process_input(body, input_format)
         result = ac.run(parsed_body, request_id, manage_by, monitor_by)
-        format_and_echo(result, verbose=verbose)
+        format_and_echo(result, output_format.get_dumper(), verbose=verbose)
     return None
 
 
@@ -105,6 +147,14 @@ def action_status(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
+    output_format: ActionOutputFormat = typer.Option(
+        ActionOutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Query an Action's status by its ACTION_ID.
@@ -112,7 +162,7 @@ def action_status(
     ac = create_action_client(action_url, action_scope)
     if ac:
         result = ac.status(action_id)
-        format_and_echo(result, verbose=verbose)
+        format_and_echo(result, output_format.get_dumper(), verbose=verbose)
     return None
 
 
@@ -131,6 +181,14 @@ def action_cancel(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
+    output_format: ActionOutputFormat = typer.Option(
+        ActionOutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Terminate a running Action by its ACTION_ID.
@@ -138,7 +196,7 @@ def action_cancel(
     ac = create_action_client(action_url, action_scope)
     if ac:
         result = ac.cancel(action_id)
-        format_and_echo(result, verbose=verbose)
+        format_and_echo(result, output_format.get_dumper(), verbose=verbose)
     return None
 
 
@@ -157,6 +215,14 @@ def action_release(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
+    output_format: ActionOutputFormat = typer.Option(
+        ActionOutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Remove an Action's execution history by its ACTION_ID.
@@ -164,7 +230,7 @@ def action_release(
     ac = create_action_client(action_url, action_scope)
     if ac:
         result = ac.release(action_id)
-        format_and_echo(result, verbose=verbose)
+        format_and_echo(result, output_format.get_dumper(), verbose=verbose)
     return None
 
 
