@@ -1,21 +1,25 @@
 import json
 from enum import Enum
-from typing import List
+from typing import Any, List, Mapping
 
 import typer
+import yaml
 from globus_sdk import GlobusHTTPResponse
 
 from globus_automate_client.cli.callbacks import (
+    flow_input_validator,
     flows_endpoint_envvar_callback,
-    json_validator_callback,
+    input_validator_callback,
     principal_or_all_authenticated_users_validator,
     principal_or_public_validator,
     principal_validator,
     url_validator_callback,
 )
+from globus_automate_client.cli.constants import InputFormat
 from globus_automate_client.cli.helpers import (
     display_http_details,
     format_and_echo,
+    process_input,
     verbosity_option,
 )
 from globus_automate_client.client_helpers import create_flows_client
@@ -42,6 +46,7 @@ class FlowDisplayFormat(str, Enum):
     json = "json"
     graphviz = "graphviz"
     image = "image"
+    yaml = "yaml"
 
 
 class ActionRole(str, Enum):
@@ -57,6 +62,24 @@ class ActionStatus(str, Enum):
     inactive = "INACTIVE"
 
 
+def _process_flow_input(flow_input: str, input_format) -> Mapping[str, Any]:
+    flow_input_dict = {}
+
+    if flow_input is not None:
+        if input_format is InputFormat.json:
+            try:
+                flow_input_dict = json.loads(flow_input)
+            except json.JSONDecodeError as e:
+                raise typer.BadParameter(f"Invalid JSON for input schema: {e}")
+        elif input_format is InputFormat.yaml:
+            try:
+                flow_input_dict = yaml.safe_load(flow_input)
+            except yaml.YAMLError as e:
+                raise typer.BadParameter(f"Invalid YAML for input schema: {e}")
+
+    return flow_input_dict
+
+
 app = typer.Typer(short_help="Manage Globus Automate Flows")
 
 
@@ -76,11 +99,11 @@ def flow_deploy(
     definition: str = typer.Option(
         ...,
         help=(
-            "JSON representation of the Flow to deploy. May be provided as a filename "
-            "or a raw JSON string."
+            "JSON or YAML representation of the Flow to deploy. May be provided as a filename "
+            "or a raw string representing a JSON object or YAML definition."
         ),
         prompt=True,
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     subtitle: str = typer.Option(
         None,
@@ -92,12 +115,12 @@ def flow_deploy(
     input_schema: str = typer.Option(
         None,
         help=(
-            "A JSON representation of a JSON Schema which will be used to "
+            "A JSON or YAML representation of a JSON Schema which will be used to "
             "validate the input to the deployed Flow when it is run. "
             "If not provided, no validation will be performed on Flow input. "
-            "May be provided as a filename or a raw JSON string."
+            "May be provided as a filename or a raw string."
         ),
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     keywords: List[str] = typer.Option(
         None,
@@ -139,16 +162,23 @@ def flow_deploy(
         callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
+    input_format: InputFormat = typer.Option(
+        InputFormat.json,
+        "--input",
+        "-i",
+        help="Input format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Deploy a new Flow.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
-    flow_dict = json.loads(definition)
-    if input_schema is not None:
-        input_schema_dict = json.loads(input_schema)
-    else:
-        input_schema_dict = None
+
+    flow_dict = process_input(definition, input_format)
+    input_schema_dict = process_input(input_schema, input_format, " for input schema")
+
     result = fc.deploy_flow(
         flow_dict,
         title,
@@ -161,7 +191,12 @@ def flow_deploy(
         input_schema_dict,
         validate_definition=validate,
     )
-    format_and_echo(result, verbose=verbose)
+
+    # Match up output format with input format
+    if input_format is InputFormat.json:
+        format_and_echo(result, json.dumps, verbose=verbose)
+    elif input_format is InputFormat.yaml:
+        format_and_echo(result, yaml.dump, verbose=verbose)
 
 
 @app.command("update")
@@ -171,10 +206,10 @@ def flow_update(
     definition: str = typer.Option(
         None,
         help=(
-            "JSON representation of the Flow to update. May be provided as a filename "
-            "or a raw JSON string."
+            "JSON or YAML representation of the Flow to update. May be provided as a filename "
+            "or a raw string."
         ),
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     subtitle: str = typer.Option(
         None,
@@ -186,12 +221,12 @@ def flow_update(
     input_schema: str = typer.Option(
         None,
         help=(
-            "A JSON representation of a JSON Schema which will be used to "
+            "A JSON or YAML representation of a JSON Schema which will be used to "
             "validate the input to the deployed Flow when it is run. "
             "If not provided, no validation will be performed on Flow input. "
-            "May be provided as a filename or a raw JSON string."
+            "May be provided as a filename or a raw string."
         ),
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     keywords: List[str] = typer.Option(
         None,
@@ -233,16 +268,22 @@ def flow_update(
         callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
+    input_format: InputFormat = typer.Option(
+        InputFormat.json,
+        "--input",
+        "-i",
+        help="Input format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Update a Flow.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
-    flow_dict = json.loads(definition)
-    if input_schema is not None:
-        input_schema_dict = json.loads(input_schema)
-    else:
-        input_schema_dict = None
+    flow_dict = process_input(definition, input_format)
+    input_schema_dict = process_input(input_schema, input_format, " for input schema")
+
     result = fc.update_flow(
         flow_id,
         flow_dict,
@@ -257,7 +298,11 @@ def flow_update(
         validate_definition=validate,
     )
     if result is not None:
-        format_and_echo(result, verbose=verbose)
+        # Match up output format with input format
+        if input_format is InputFormat.json:
+            format_and_echo(result, json.dumps, verbose=verbose)
+        elif input_format is InputFormat.yaml:
+            format_and_echo(result, yaml.dump, verbose=verbose)
     else:
         print("No operation to perform")
 
@@ -267,11 +312,11 @@ def flow_lint(
     definition: str = typer.Option(
         ...,
         help=(
-            "JSON representation of the Flow to deploy. May be provided as a filename "
-            "or a raw JSON string."
+            "JSON or YAML representation of the Flow to deploy. May be provided as a filename "
+            "or a raw string."
         ),
         prompt=True,
-        callback=json_validator_callback,
+        callback=input_validator_callback,
     ),
     validate: bool = typer.Option(
         True,
@@ -287,24 +332,36 @@ def flow_lint(
         case_sensitive=False,
         show_default=True,
     ),
+    input_format: InputFormat = typer.Option(
+        InputFormat.json,
+        "--input",
+        "-i",
+        help="Input format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Parse and validate a Flow definition by providing visual output.
     """
-    flow_dict = json.loads(definition)
+    flow_dict = process_input(definition, input_format)
+
     try:
         if validate:
             validate_flow_definition(flow_dict)
     except FlowValidationError as fve:
         typer.secho(str(fve), fg=typer.colors.RED)
         raise typer.Exit(code=1)
+
     graph = graphviz_format(flow_dict)
-    if output_format == FlowDisplayFormat.json:
+    if output_format is FlowDisplayFormat.json:
         format_and_echo(flow_dict)
-    elif output_format == FlowDisplayFormat.graphviz:
+    elif output_format is FlowDisplayFormat.yaml:
+        format_and_echo(flow_dict, yaml.dump)
+    elif output_format is FlowDisplayFormat.graphviz:
         typer.echo(graph.source)
     else:
-        graph.render(f"flows-output/graph", view=True, cleanup=True)
+        graph.render("flows-output/graph", view=True, cleanup=True)
 
 
 @app.command("list")
@@ -337,6 +394,14 @@ def flow_list(
         callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
+    output_format: FlowDisplayFormat = typer.Option(
+        FlowDisplayFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     List Flows for which you have access.
@@ -345,7 +410,8 @@ def flow_list(
     flows = fc.list_flows(
         roles=[r.value for r in roles], marker=marker, per_page=per_page
     )
-    format_and_echo(flows, verbose=verbose)
+
+    _format_and_display_flow(flows, output_format, verbose=verbose)
 
 
 @app.command("display")
@@ -372,6 +438,7 @@ def flow_display(
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     flow_get = fc.get_flow(flow_id)
+
     _format_and_display_flow(flow_get, output_format, verbose=verbose)
 
 
@@ -408,10 +475,10 @@ def flow_run(
     flow_input: str = typer.Option(
         None,
         help=(
-            "JSON formatted input to the Flow. May be provided as a filename "
-            "or a raw JSON string."
+            "JSON or YAML formatted input to the Flow. May be provided as a filename "
+            "or a raw string."
         ),
-        callback=json_validator_callback,
+        callback=flow_input_validator,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -424,17 +491,31 @@ def flow_run(
         callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
+    output_format: FlowDisplayFormat = typer.Option(
+        FlowDisplayFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
+    input_format: InputFormat = typer.Option(
+        InputFormat.json,
+        "--input",
+        "-i",
+        help="Input format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
 ):
     """
     Run an instance of a Flow. The argument provides the initial state of the Flow.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
-    if flow_input is not None:
-        flow_input_dict = json.loads(flow_input)
-    else:
-        flow_input_dict = {}
+    flow_input_dict = _process_flow_input(flow_input, input_format)
+
     response = fc.run_flow(flow_id, flow_scope, flow_input_dict)
-    format_and_echo(response, verbose=verbose)
+    _format_and_display_flow(response, output_format, verbose=verbose)
 
 
 @app.command("action-list")
@@ -654,8 +735,8 @@ def flow_action_log(
     if verbose:
         display_http_details(resp)
 
-    if output_format == FlowDisplayFormat.json:
-        format_and_echo(resp)
+    if output_format in (FlowDisplayFormat.json, FlowDisplayFormat.yaml):
+        _format_and_display_flow(resp, output_format, verbose)
     elif output_format in (FlowDisplayFormat.graphviz, FlowDisplayFormat.image):
         flow_def_resp = fc.get_flow(flow_id)
         flow_def = flow_def_resp.data["definition"]
@@ -665,7 +746,7 @@ def flow_action_log(
         if output_format == FlowDisplayFormat.graphviz:
             typer.echo(graphviz_out.source)
         else:
-            graphviz_out.render(f"flows-output/graph", view=True, cleanup=True)
+            graphviz_out.render("flows-output/graph", view=True, cleanup=True)
 
 
 def _format_and_display_flow(
@@ -677,14 +758,16 @@ def _format_and_display_flow(
     if verbose:
         display_http_details(flow_resp)
 
-    if output_format == FlowDisplayFormat.json:
-        format_and_echo(flow_resp)
+    if output_format is FlowDisplayFormat.json:
+        format_and_echo(flow_resp, json.dumps)
+    elif output_format is FlowDisplayFormat.yaml:
+        format_and_echo(flow_resp, yaml.dump)
     elif output_format in (FlowDisplayFormat.graphviz, FlowDisplayFormat.image):
         graphviz_out = graphviz_format(flow_resp.data["definition"])
         if output_format == FlowDisplayFormat.graphviz:
             typer.echo(graphviz_out.source)
         else:
-            graphviz_out.render(f"flows-output/graph", view=True, cleanup=True)
+            graphviz_out.render("flows-output/graph", view=True, cleanup=True)
 
 
 if __name__ == "__main__":
